class Bus:
    """ This classes stores the information pertaining to a bus """

    def __init__(self):
        """
        Args: Self
        Behavior: Initializes the attributes of a bus
        """
        self.station = 0         # the position of the bus
        self.capacity = 5        # the capacity of the bus
        self.passengers = []     # the content of the bus
        self.terminus = 5        # The last station

    def move(self):
        """
        Args: Self
        Behavior: Increases station by 1, unless we're at the terminus
        """        
        if self.station< self.terminus:
            self.station+=1
    
    def unload(self):
        """
        Args: Self
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """ 
        unloaded = [dest for dest in self.passengers if dest==self.station]
        self.passengers = [dest for dest in self.passengers if dest!=self.station]
        return len(unloaded)
    
    def load(self, waiting_line):
        """
        Args: Self, and waiting_line, the list of people waiting to board
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """         
        number_boarding = min(len(waiting_line),self.capacity-len(self.passengers))
        people_boarding = waiting_line[0:number_boarding]
        self.passengers.extend(people_boarding)
        return number_boarding
    
    def __str__(self):
        """
        Args: Self, 
        Returns: String describing bus
        """           
        return "Bus at station "+str(self.station) + \
               " contains passengers " + str(self.passengers)
    

class PayBus(Bus):
    def __init__(self, price=2):
        Bus.__init__(self)
        self.cost_of_ticket = price # cost of a ticket
        self.cash = 0         # the total cash onboard
        
    def load(self, waiting_line):
        number_boarding = Bus.load(self, waiting_line)
        self.cash += number_boarding * self.cost_of_ticket
        return number_boarding
    
    def __str__(self):         
        return Bus.__str__(self)+\
               ". Amount of cash: "+ str(self.cash)
    

stm_bus = PayBus(2)
stm_bus.load([3,4,5,2,6,2,3])
stm_bus.station = 3
stm_bus.cash = 134
print(stm_bus) 
# Prints: Bus at station 3 contains passengers [3, 4, 5, 2, 6]. 

stm_bus = PayBus(2)
stm_bus.load([3,4,5,2,6,2,3])
print("Cash = ",stm_bus.cash) # Prints Cash = 10

generic_bus = Bus()

stm_bus = PayBus(2)
print(generic_bus)
print(stm_bus)
stm_bus.load([4,2,5,3,6,4,2,4])
print(stm_bus)

class ExpressBus(Bus):
    def __init__(self, my_stops):
        Bus.__init__(self)
        self.stops = my_stops # list of stations 
                              # where the bus will stop

    def unload(self):
        if self.station in self.stops:
            return Bus.unload(self) # allow unloading
        else:
            return []               # no unloading
    
    def load(self, waiting_line):
        if self.station in self.stops:  # allow loading
            return Bus.load(self, waiting_line) 
        else:                           # no loading
            return 0                            

    def load_safe(self, waiting_line):
        # only allows boarding for passengers 
        # whose destination is one of the stops 
        # of the express bus
        should_board = [p for p in waiting_line \
                        if p in self.stops]
        number_boarding = min(len(should_board),\
                    self.capacity -len(self.passengers))
        people_boarding = should_board[0:number_boarding]
        self.passengers.extend(people_boarding)
        return number_boarding        
        
        
exp = ExpressBus([0,2,4]) # bus will stop only at 0, 2, 4
slow = Bus()
exp.load([5,3,1]) 
slow.load([5,3,1]) 
print(exp) # Bus at station 0 has passengers [5, 3, 1]
print(slow) # Bus at station 0 has passengers [5, 3, 1]

exp.move()
slow.move()
exp.load([4,3])  # Nobody gets loaded onto express bus
slow.load([4,3])  # But passengers can board the slow bus
print(exp) # Bus at station 1 has passengers [5, 3, 1]
print(slow) # Bus at station 1 has passengers [5, 3, 1, 4, 3]

exp = ExpressBus([0,2,4])
exp.load_safe([4,2,3,1,3,2])
print(exp)  # Bus at station 0 has passengers [4, 2, 2]
slow = Bus()
slow.load_safe([4,2,3,1,3,2])  
# AttributeError: 'Bus' object has no attribute 'load_safe'

