import matplotlib.pyplot as plt

class Bus:
    """ This classes stores the information pertaining to a bus """

    def __init__(self):
        """
        Args: Self
        Behavior: Initializes the attributes of a bus
        """
        self.station = 0         # the position of the bus
        self.capacity = 5        # the capacity of the bus
        self.passengers = []     # the content of the bus
        self.terminus = 5        # The last station

    def move(self):
        """
        Args: Self
        Behavior: Increases station by 1, unless we're at the terminus
        """        
        if self.station< self.terminus:
            self.station+=1
    
    def unload(self):
        """
        Args: Self
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """ 
        unloaded = [dest for dest in self.passengers if dest==self.station]
        self.passengers = [dest for dest in self.passengers if dest!=self.station]
        return len(unloaded)
    
    def load(self, waiting_line):
        """
        Args: Self, and waiting_line, the list of people waiting to board
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """         
        number_boarding = min(len(waiting_line),self.capacity-len(self.passengers))
        people_boarding = waiting_line[0:number_boarding]
        self.passengers.extend(people_boarding)
        return number_boarding
    
    def __str__(self):
        """
        Args: Self, 
        Returns: String describing bus
        """           
        return "Bus at station "+str(self.station) + \
               " contains passengers " + str(self.passengers)
    

class PayBus(Bus):
    def __init__(self, ticket_price):
        """
        Args: Self
        Behavior: Initializes the attributes of a PayBus
        """
        self.cash = 0         # the total amount of cash onboard the bus
        self.cost_of_ticket = ticket_price # cost of a ticket
        Bus.__init__(self)
        
    def load(self, waiting_line):
        """
        Args: Self, and waiting_line, the list of people waiting to board
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """         
        number_boarding = Bus.load(self, waiting_line)
        self.cash += number_boarding * self.cost_of_ticket
        return number_boarding
    
    def __str__(self):
        """
        Args: Self, 
        Returns: String describing PayBus
        """           
        return Bus.__str__(self)+". Amount of cash onboard: "+ str(self.cash)
    


generic_bus = Bus()

stm_bus = PayBus(2)
print(generic_bus)
print(stm_bus)
stm_bus.load([4,2,5,3,6,4,2,4])
print(stm_bus)

class ExpressBus(Bus):
    def __init__(self, my_stops):
        Bus.__init__(self)
        self.stops = my_stops

    def unload(self):
        """
        Args: Self
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """ 
        if self.station in self.stops:
            return Bus.unload(self) 
        else:
            return []
    
    def load(self, waiting_line):
        """
        Args: Self, and waiting_line, the list of people waiting to board
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """         
        if self.station in self.stops:
            return Bus.load(self, waiting_line)
        else:
            return 0

exp = ExpressBus([0,2,4]) # bus will stop only at 0, 2, 4
exp.load([5,3,1])
print(exp)

exp.move()
exp.load([4,3])
print(exp)

exp.move()
exp.load([5,4,4])
print(exp)

Bus.move(exp)
print(exp)

# Note: passenger who wanted to get off at station 1 will not be able to.
# modify unload function for ExpressBus so that passengers get off
# the bus if they have reached their stop or beyond.
