class Bus:
    """ This classes stores the information pertaining to a bus """

    def __init__(self):
        """
        Args: Self
        Behavior: Initializes the attributes of a bus
        """
        self.station = 0         # the position of the bus
        self.capacity = 5        # the capacity of the bus
        self.passengers = []     # the content of the bus
        self.terminus = 5        # The last station

    def move(self):
        """
        Args: Self
        Behavior: Increases station by 1, unless we're at the terminus
        """        
        if self.station< self.terminus:
            self.station+=1
    
    def unload(self):
        """
        Args: Self
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """ 
        unloaded = [dest for dest in self.passengers if dest==self.station]
        self.passengers = [dest for dest in self.passengers if dest!=self.station]
        return len(unloaded)
    
    def load(self, waiting_line):
        """
        Args: Self, and waiting_line, the list of people waiting to board
        Behavior: Removes passengers who have reached their station
        Returns: number of passengers who disembark
        """         
        number_boarding = min(len(waiting_line),self.capacity-len(self.passengers))
        people_boarding = waiting_line[0:number_boarding]
        self.passengers.extend(people_boarding)
        return number_boarding
    
    def __str__(self):
        """
        Args: Self, 
        Returns: String describing bus
        """           
        return "Bus at station "+str(self.station) + \
               " contains passengers " + str(self.passengers)
