from bus_generic import Bus

class ExpressBus(Bus):
    def __init__(self, my_stops):
        Bus.__init__(self)
        self.stops = my_stops # list of stations 
                              # where the bus will stop

    def unload(self):
        if self.station in self.stops:
            return Bus.unload(self) # allow unloading
        else:
            return []               # no unloading
    
    def load(self, waiting_line):
        if self.station in self.stops:  # allow loading
            return Bus.load(self, waiting_line) 
        else:                           # no loading
            return 0                            

    def load_safe(self, waiting_line):
        # allows passengers to board only if 
        # their destinations are among the express bus stops 
        should_board = [p for p in waiting_line \
                        if p in self.stops]
        number_boarding = min(len(should_board),\
                    self.capacity -len(self.passengers))
        people_boarding = should_board[0:number_boarding]
        self.passengers.extend(people_boarding)
        return number_boarding        
        
        
exp = ExpressBus([0,2,4]) # bus will stop only at 0, 2, 4
slow = Bus()
exp.load([5,3,1]) 
slow.load([5,3,1]) 
print(exp) # Bus at station 0 has passengers [5, 3, 1]
print(slow) # Bus at station 0 has passengers [5, 3, 1]

exp.move()
slow.move()
exp.load([4,3])  # Nobody gets loaded onto express bus
slow.load([4,3])  # But passengers can board the slow bus
print(exp)  # Bus at station 1 has passengers [5, 3, 1]
print(slow) # Bus at station 1 has passengers [5, 3, 1, 4, 3]

exp = ExpressBus([0,2,4])
exp.load_safe([4,2,3,1,3,2])
print(exp)  # Bus at station 0 has passengers [4, 2, 2]
slow = Bus()
#slow.load_safe([4,2,3,1,3,2])  
# AttributeError: 'Bus' object has no attribute 'load_safe'